@@ -55,6 +55,7 @@
 #include "camel-mapi-folder.h"
 #include "camel-mapi-private.h"
 #include "camel-mapi-summary.h"
+#include "camel-mapi-utils.h"
 
 #define DEBUG_FN( ) printf("----%u %s\n", (unsigned int)pthread_self(), __FUNCTION__);
 #define SUMMARY_FETCH_BATCH_COUNT 150
@@ -645,7 +646,7 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	guint32 index, count, options = 0;
 	GSList *server_uid_list = NULL;
 	const gchar *uid = NULL;
-	gchar *tmp = NULL;
+	GSList *tmp_list_item = NULL;
 
 	/* Currently we don't have simple wrapper over getprops.*/
 	const guint32 prop_list[] = { PR_LAST_MODIFICATION_TIME };
@@ -690,12 +691,12 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 
 		if (server_uid_list) {
 			/* TODO : Find a better way and avoid this linear search */
-			tmp = g_slist_find_custom (server_uid_list, (const gchar *)uid, 
+			tmp_list_item = g_slist_find_custom (server_uid_list, (gconstpointer) uid, 
 						  (GCompareFunc) g_strcmp0);
 		}
 
 		/* If it is not in server list, clean our cache */
-		if (!tmp && uid) {
+		if (!tmp_list_item->data && uid) {
 			CAMEL_MAPI_FOLDER_REC_LOCK (m->folder, cache_lock);
 			camel_folder_summary_remove_uid (m->folder->summary, uid);
 			camel_data_cache_remove (mapi_folder->cache, "cache", uid, NULL);
@@ -1274,7 +1275,7 @@ fetch_item_cb (FetchItemsCallbackData *item_data, gpointer data)
 		body = g_new0(ExchangeMAPIStream, 1);
 		body->proptag = PR_BODY;
 		body->value = g_byte_array_new ();
-		body->value = g_byte_array_append (body->value, appointment_body_str, g_utf8_strlen ((const gchar *)appointment_body_str, -1));
+		body->value = g_byte_array_append (body->value, appointment_body_str, strlen ((const char *)appointment_body_str));
 
 		item->msg.body_parts = g_slist_append (item->msg.body_parts, body);
 
@@ -1907,6 +1908,57 @@ mapi_cmp_uids (CamelFolder *folder, const char *uid1, const char *uid2)
 	return strcmp (uid1, uid2);
 }
 
+static void
+mapi_append_message (CamelFolder *folder, CamelMimeMessage *message,
+		const CamelMessageInfo *info, gchar **appended_uid,
+		CamelException *ex)
+{
+	CamelMapiStore *mapi_store= CAMEL_MAPI_STORE(folder->parent_store);
+	CamelOfflineStore *offline = (CamelOfflineStore *) folder->parent_store;
+	CamelAddress *from = NULL, *recipients = NULL;
+	CamelStoreInfo *si;
+
+	MapiItem *item = NULL;
+	mapi_id_t fid = 0, mid = 0;
+	const gchar *folder_id;
+
+	/*Reject outbox / sent & trash*/
+	si = camel_store_summary_path ((CamelStoreSummary *)mapi_store->summary,
+				       folder->full_name);
+
+	if (si->flags & CAMEL_FOLDER_TYPE_TRASH ||
+	    si->flags & CAMEL_FOLDER_TYPE_OUTBOX) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, 
+				      _("Cannot append message to folder '%s'"),
+				      folder->full_name);
+		return;
+	}
+
+	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM, 
+				      _("Offline. '%s'"));
+		return;
+	}
+
+	folder_id =  camel_mapi_store_folder_id_lookup (mapi_store,
+							folder->full_name);
+
+	exchange_mapi_util_mapi_id_from_string (folder_id, &fid);
+
+	/* Convert MIME to Item */
+	from = (CamelAddress *) camel_mime_message_get_from (message);
+	recipients = (CamelAddress *) camel_mime_message_get_recipients (message, "to");
+
+	item = camel_mapi_utils_mime_to_item (message, from, recipients, ex);
+
+	mid = exchange_mapi_create_item (olFolderOutbox, fid, NULL, NULL, 
+					 camel_mapi_utils_create_item_build_props, item,
+					 item->recipients, item->attachments,
+					 item->generic_streams, 0);
+
+	*appended_uid = exchange_mapi_util_mapi_ids_to_uid(fid, mid);
+}
+
 static void
 camel_mapi_folder_class_init (CamelMapiFolderClass *camel_mapi_folder_class)
 {
@@ -1923,7 +1975,7 @@ camel_mapi_folder_class_init (CamelMapiFolderClass *camel_mapi_folder_class)
 /* 	camel_folder_class->get_message_info = mapi_get_message_info; */
 /* 	camel_folder_class->search_by_uids = mapi_folder_search_by_uids;  */
 	camel_folder_class->search_free = mapi_folder_search_free;
-/* 	camel_folder_class->append_message = mapi_append_message; */
+	camel_folder_class->append_message = mapi_append_message;
 	camel_folder_class->refresh_info = mapi_refresh_info;
 	camel_folder_class->sync = mapi_sync;
 	camel_folder_class->expunge = mapi_expunge;
@@ -1974,7 +2026,8 @@ camel_mapi_folder_get_type (void)
 }
 
 CamelFolder *
-camel_mapi_folder_new(CamelStore *store, const char *folder_name, const char *folder_dir, guint32 flags, CamelException *ex)
+camel_mapi_folder_new (CamelStore *store, const char *folder_name, const char *folder_dir,
+		      guint32 flags, CamelException *ex)
 {
 
 	CamelFolder	*folder = NULL;
@@ -2030,10 +2083,8 @@ camel_mapi_folder_new(CamelStore *store, const char *folder_name, const char *fo
 /* 		return NULL; */
 /* 	} */
 
-	if (!strcmp (folder_name, "Mailbox")) {
-		if (camel_url_get_param (((CamelService *) store)->url, "filter"))
-			folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
-	}
+	if (camel_url_get_param (((CamelService *) store)->url, "filter"))
+		folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
 
 	mapi_folder->search = camel_folder_search_new ();
 	if (!mapi_folder->search) {
